/*
 * Copyright (c) 2000 Murray Stokely
 * Copyright (c) 2000 BSDi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer
 *    in this position and unchanged.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software withough specific prior written permission
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Test program for `ni' device driver for National Instruments data
 * acquisition cards.
 *
 * References :
 *  [1] DAQ-STC Technical Reference Manual, January 1999 Edition
 *  [2] PCI E Series Register-Level Programmer Manual, November 1998 Edition
 *  [3] PCI E Series User Manual, January 1999 Edition
 *
 * Murray Stokely
 * 19 Nov 2000 
 */

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <string.h>

#include "niio.h"

float
toCelsius(float voltage) {
    return (voltage * 100);
}

float
toFarenheit(float voltage) {
    return ((voltage * 100) * 9/5) + 32;
}

/* 
 * From [2] 4-9
 * Acquires one sample from channel 0 
 */

void
Example_One() {
    int fd;
    int16_t in;
    float voltage;
    struct ai_channel chan;

    printf("-------------------------------------------------------\n"
	   " Example One (Analog Input - One sample)\n"
	   "-------------------------------------------------------\n\n");

    if ((fd = open("/dev/nidaq0",O_RDONLY)) < 0) {
	printf("Couldn't open!\n");
	return;
    }

    printf("Reading channel 0 in differential mode\n");
    bzero(&chan, sizeof(chan));
    chan.num  = 0;
    chan.mode = DIFF;
    chan.gain = 1;

    if (ioctl(fd, NICTL_AI_CONVERT, &chan)) printf("Could not start conversion!\n");

    sleep(1);

    if ((read(fd, &in, sizeof(in)))< sizeof(in)) {
	perror("cant read nidaq"); 
	exit(2);
    }
    printf("We read in : %d\n", in);
    voltage = (float)in * 10 / 2048; /* Get capabilities here? */
    printf("%5.2fV\t%5.2fdeg C\t%5.2fdeg F\n", voltage, 
	   toCelsius(voltage), toFarenheit(voltage));
    return;
}

void
Example_Two() {
    int fd, i, in, nr;
    float voltage;
    uint16_t valin[100000];
    int nscans           = 10, 
	channels        = 2,
	scan_interval   = 50,
	sample_interval = 50;
    struct ai_channel chan;
    struct ai_scaninfo scans;

    printf("-------------------------------------------------------\n"
	   " Example Two (Analog Input)\n"
	   "-------------------------------------------------------\n\n");
    /* Step 1 - done by driver at initialization */
    if ((fd = open("/dev/nidaq0",O_RDONLY)) < 0) {
	printf("Couldn't open!\n");
	return;
    }

    /* Step 2 -  Add the channels */
    bzero(&chan, sizeof(chan));
    chan.num  = 0;
    chan.mode = DIFF;
    chan.gain = 1;
    if (ioctl(fd, NICTL_AI_ADDCH, &chan)) {
	printf("Could not add channel!\n");
	exit(2);
    }

    bzero(&chan, sizeof(chan));
    chan.num  = 1;
    chan.mode = DIFF;
    chan.gain = 1;
    if (ioctl(fd, NICTL_AI_ADDCH, &chan)) {
	printf("Could not add channel!\n");
	exit(2);
    }
    

    /* Set the scans to 1 (just one sample from one channel) */
    bzero(&scans, sizeof(scans));
    scans.scans           = 10;
    scans.scan_interval   = 50;
    scans.sample_interval = 50;

    if (ioctl(fd, NICTL_AI_SCAN_SETUP, &scans)) {
	printf("Could not setup scan!\n");
	exit(2);
    }
    if (ioctl(fd, NICTL_AI_SCAN_START, &scans)) {
	printf("Could not start scan!\n");
	exit(2);
    }

    sleep(2);

    if((nr=read(fd, valin, scans.scans *2* 2))<1){
	perror("cant read nidaq"); exit(2);}
    printf("We read in : %d\n", valin[0]);
    printf("requested : %d read = %d\n", scans.scans * 2 * 2, nr);
    for(i=0;i<nr/2;i++) {
	voltage = (float)valin[i] * 10 / 2048;
	printf("%d %5.2fV\t%5.2fdeg C\t%5.2fdeg F\n", valin[i], voltage, 
	   toCelsius(voltage), toFarenheit(voltage));
    }

    close(fd);
}

int 
main() {

    int i,nr;
    unsigned char out=0, in=0;


    /* Write tricky userland code here to find all NI devices! */
    /*    Example_One(); */
    Example_One();
    Example_Two();
    return 0;
}
